#!/usr/bin/env python3
"""
Compliance Checker
Advanced security compliance assessment and reporting tool.
Parameters:
    - target: Target system or configuration
    - standard: Compliance standard to check against
    - output: Output file for results
Requires: psutil, winreg (Windows), pwd, grp (Linux) packages
"""
import argparse
import logging
from datetime import datetime
import json
import os
import sys
import platform
import subprocess
import socket
from pathlib import Path
import psutil
try:
    import winreg
except ImportError:
    winreg = None

class ComplianceChecker:
    def __init__(self, target, standard='pci-dss'):
        self.target = target
        self.standard = standard.lower()
        self.os_type = platform.system().lower()
        self.results = {}
        self.setup_logging()
        self.load_standards()

    def setup_logging(self):
        logging.basicConfig(
            level=logging.INFO,
            format='%(asctime)s - %(levelname)s - %(message)s'
        )
        self.logger = logging.getLogger(__name__)

    def load_standards(self):
        """Load compliance standards"""
        self.standards = {
            'pci-dss': {
                'requirement_1': {
                    'name': 'Install and maintain a firewall configuration',
                    'checks': [
                        'firewall_installed',
                        'default_deny_policy',
                        'documented_rules'
                    ]
                },
                'requirement_2': {
                    'name': 'Do not use vendor-supplied defaults',
                    'checks': [
                        'default_passwords',
                        'unnecessary_services',
                        'secure_protocols'
                    ]
                },
                'requirement_3': {
                    'name': 'Protect stored cardholder data',
                    'checks': [
                        'encryption_used',
                        'key_management',
                        'secure_storage'
                    ]
                },
                'requirement_4': {
                    'name': 'Encrypt transmission of cardholder data',
                    'checks': [
                        'transmission_encryption',
                        'trusted_keys',
                        'secure_protocols'
                    ]
                }
            },
            'hipaa': {
                'privacy_rule': {
                    'name': 'Privacy of health information',
                    'checks': [
                        'data_access_controls',
                        'minimum_disclosure',
                        'patient_rights'
                    ]
                },
                'security_rule': {
                    'name': 'Security of health information',
                    'checks': [
                        'access_control',
                        'audit_controls',
                        'integrity_controls'
                    ]
                },
                'enforcement_rule': {
                    'name': 'Enforcement and penalties',
                    'checks': [
                        'incident_response',
                        'breach_notification',
                        'documentation'
                    ]
                }
            },
            'iso27001': {
                'information_security': {
                    'name': 'Information security policies',
                    'checks': [
                        'policy_documented',
                        'management_commitment',
                        'policy_review'
                    ]
                },
                'asset_management': {
                    'name': 'Asset management',
                    'checks': [
                        'asset_inventory',
                        'acceptable_use',
                        'information_classification'
                    ]
                },
                'access_control': {
                    'name': 'Access control',
                    'checks': [
                        'access_policy',
                        'user_registration',
                        'privilege_management'
                    ]
                }
            }
        }

    def check_pci_compliance(self):
        """Check PCI DSS compliance"""
        results = {
            'requirement_1': {},
            'requirement_2': {},
            'requirement_3': {},
            'requirement_4': {}
        }

        try:
            # Check Requirement 1: Firewall
            results['requirement_1'] = self._check_firewall_compliance()

            # Check Requirement 2: Default Settings
            results['requirement_2'] = self._check_default_settings()

            # Check Requirement 3: Data Protection
            results['requirement_3'] = self._check_data_protection()

            # Check Requirement 4: Transmission Security
            results['requirement_4'] = self._check_transmission_security()

        except Exception as e:
            self.logger.error(f"Error checking PCI compliance: {str(e)}")

        return results

    def _check_firewall_compliance(self):
        """Check firewall compliance"""
        results = {}
        try:
            # Check firewall installation
            if self.os_type == 'windows':
                results['firewall_installed'] = self._check_windows_firewall()
            else:
                results['firewall_installed'] = self._check_linux_firewall()

            # Check default deny policy
            results['default_deny_policy'] = self._check_default_deny()

            # Check rule documentation
            results['documented_rules'] = self._check_rule_documentation()

        except Exception as e:
            self.logger.error(f"Error checking firewall: {str(e)}")

        return results

    def _check_windows_firewall(self):
        """Check Windows firewall status"""
        try:
            result = subprocess.run(
                ['netsh', 'advfirewall', 'show', 'allprofiles'],
                capture_output=True,
                text=True
            )
            return 'ON' in result.stdout
        except:
            return None

    def _check_linux_firewall(self):
        """Check Linux firewall status"""
        try:
            result = subprocess.run(
                ['iptables', '-L'],
                capture_output=True,
                text=True
            )
            return result.returncode == 0
        except:
            return None

    def _check_default_deny(self):
        """Check default deny policy"""
        try:
            if self.os_type == 'windows':
                result = subprocess.run(
                    ['netsh', 'advfirewall', 'show', 'currentprofile'],
                    capture_output=True,
                    text=True
                )
                return 'Block' in result.stdout
            else:
                result = subprocess.run(
                    ['iptables', '-L', 'INPUT'],
                    capture_output=True,
                    text=True
                )
                return 'DROP' in result.stdout or 'REJECT' in result.stdout
        except:
            return None

    def _check_rule_documentation(self):
        """Check firewall rule documentation"""
        try:
            if self.os_type == 'windows':
                rules_file = 'C:\\Windows\\System32\\LogFiles\\Firewall\\pfirewall.log'
            else:
                rules_file = '/etc/iptables/rules.v4'
            
            return os.path.exists(rules_file)
        except:
            return None

    def _check_default_settings(self):
        """Check default security settings"""
        results = {}
        try:
            # Check default passwords
            results['default_passwords'] = self._check_password_policies()

            # Check unnecessary services
            results['unnecessary_services'] = self._check_services()

            # Check secure protocols
            results['secure_protocols'] = self._check_protocols()

        except Exception as e:
            self.logger.error(f"Error checking settings: {str(e)}")

        return results

    def _check_password_policies(self):
        """Check password policies"""
        try:
            if self.os_type == 'windows':
                result = subprocess.run(
                    ['net', 'accounts'],
                    capture_output=True,
                    text=True
                )
                return (
                    'Minimum password length' in result.stdout and
                    'Maximum password age' in result.stdout
                )
            else:
                with open('/etc/login.defs', 'r') as f:
                    content = f.read()
                    return (
                        'PASS_MAX_DAYS' in content and
                        'PASS_MIN_LEN' in content
                    )
        except:
            return None

    def _check_services(self):
        """Check unnecessary services"""
        try:
            if self.os_type == 'windows':
                result = subprocess.run(
                    ['net', 'start'],
                    capture_output=True,
                    text=True
                )
                return 'Telnet' not in result.stdout
            else:
                result = subprocess.run(
                    ['systemctl', 'list-units', '--type=service'],
                    capture_output=True,
                    text=True
                )
                return 'telnet' not in result.stdout.lower()
        except:
            return None

    def _check_protocols(self):
        """Check secure protocols"""
        try:
            if self.os_type == 'windows':
                result = subprocess.run(
                    ['reg', 'query', 'HKLM\\SYSTEM\\CurrentControlSet\\Control\\SecurityProviders\\SCHANNEL'],
                    capture_output=True,
                    text=True
                )
                return 'TLS' in result.stdout
            else:
                with open('/etc/ssl/openssl.cnf', 'r') as f:
                    content = f.read()
                    return 'TLSv1.2' in content or 'TLSv1.3' in content
        except:
            return None

    def _check_data_protection(self):
        """Check data protection measures"""
        results = {}
        try:
            # Check encryption usage
            results['encryption_used'] = self._check_encryption()

            # Check key management
            results['key_management'] = self._check_key_management()

            # Check secure storage
            results['secure_storage'] = self._check_secure_storage()

        except Exception as e:
            self.logger.error(f"Error checking data protection: {str(e)}")

        return results

    def _check_encryption(self):
        """Check encryption usage"""
        try:
            if self.os_type == 'windows':
                result = subprocess.run(
                    ['manage-bde', '-status'],
                    capture_output=True,
                    text=True
                )
                return 'Protection On' in result.stdout
            else:
                result = subprocess.run(
                    ['dmsetup', 'status'],
                    capture_output=True,
                    text=True
                )
                return 'crypt' in result.stdout
        except:
            return None

    def _check_key_management(self):
        """Check key management practices"""
        try:
            if self.os_type == 'windows':
                cert_store = 'cert:\\LocalMachine\\My'
                result = subprocess.run(
                    ['certutil', '-store', cert_store],
                    capture_output=True,
                    text=True
                )
                return '=====' in result.stdout
            else:
                return os.path.exists('/etc/ssl/private')
        except:
            return None

    def _check_secure_storage(self):
        """Check secure storage configuration"""
        try:
            if self.os_type == 'windows':
                result = subprocess.run(
                    ['cipher', '/s:C:\\'],
                    capture_output=True,
                    text=True
                )
                return 'encrypted' in result.stdout.lower()
            else:
                result = subprocess.run(
                    ['mount'],
                    capture_output=True,
                    text=True
                )
                return 'crypto' in result.stdout
        except:
            return None

    def _check_transmission_security(self):
        """Check transmission security measures"""
        results = {}
        try:
            # Check transmission encryption
            results['transmission_encryption'] = self._check_transmission_encryption()

            # Check trusted keys
            results['trusted_keys'] = self._check_trusted_keys()

            # Check secure protocols
            results['secure_protocols'] = self._check_secure_protocols()

        except Exception as e:
            self.logger.error(f"Error checking transmission security: {str(e)}")

        return results

    def _check_transmission_encryption(self):
        """Check transmission encryption"""
        try:
            if self.os_type == 'windows':
                result = subprocess.run(
                    ['reg', 'query', 'HKLM\\SYSTEM\\CurrentControlSet\\Control\\SecurityProviders\\SCHANNEL\\Protocols'],
                    capture_output=True,
                    text=True
                )
                return 'TLS' in result.stdout
            else:
                with open('/etc/ssl/openssl.cnf', 'r') as f:
                    content = f.read()
                    return 'TLSv1.2' in content or 'TLSv1.3' in content
        except:
            return None

    def _check_trusted_keys(self):
        """Check trusted keys configuration"""
        try:
            if self.os_type == 'windows':
                result = subprocess.run(
                    ['certutil', '-store', 'root'],
                    capture_output=True,
                    text=True
                )
                return '=====' in result.stdout
            else:
                return os.path.exists('/etc/ssl/certs')
        except:
            return None

    def _check_secure_protocols(self):
        """Check secure protocol configuration"""
        try:
            if self.os_type == 'windows':
                result = subprocess.run(
                    ['reg', 'query', 'HKLM\\SYSTEM\\CurrentControlSet\\Control\\SecurityProviders\\SCHANNEL\\Protocols'],
                    capture_output=True,
                    text=True
                )
                return (
                    'TLS 1.2' in result.stdout or
                    'TLS 1.3' in result.stdout
                )
            else:
                with open('/etc/ssl/openssl.cnf', 'r') as f:
                    content = f.read()
                    return (
                        'TLSv1.2' in content or
                        'TLSv1.3' in content
                    )
        except:
            return None

    def check_hipaa_compliance(self):
        """Check HIPAA compliance"""
        results = {
            'privacy_rule': {},
            'security_rule': {},
            'enforcement_rule': {}
        }

        try:
            # Check Privacy Rule compliance
            results['privacy_rule'] = self._check_privacy_compliance()

            # Check Security Rule compliance
            results['security_rule'] = self._check_security_compliance()

            # Check Enforcement Rule compliance
            results['enforcement_rule'] = self._check_enforcement_compliance()

        except Exception as e:
            self.logger.error(f"Error checking HIPAA compliance: {str(e)}")

        return results

    def check_iso27001_compliance(self):
        """Check ISO 27001 compliance"""
        results = {
            'information_security': {},
            'asset_management': {},
            'access_control': {}
        }

        try:
            # Check Information Security compliance
            results['information_security'] = self._check_infosec_compliance()

            # Check Asset Management compliance
            results['asset_management'] = self._check_asset_compliance()

            # Check Access Control compliance
            results['access_control'] = self._check_access_compliance()

        except Exception as e:
            self.logger.error(f"Error checking ISO compliance: {str(e)}")

        return results

    def generate_report(self):
        """Generate compliance report"""
        try:
            report = {
                'timestamp': datetime.now().isoformat(),
                'target': self.target,
                'standard': self.standard,
                'system': {
                    'os': platform.system(),
                    'version': platform.version(),
                    'architecture': platform.machine()
                }
            }

            # Get compliance check results
            if self.standard == 'pci-dss':
                self.results = self.check_pci_compliance()
            elif self.standard == 'hipaa':
                self.results = self.check_hipaa_compliance()
            elif self.standard == 'iso27001':
                self.results = self.check_iso27001_compliance()

            report['results'] = self.results

            # Calculate compliance scores
            scores = self._calculate_compliance()
            report['compliance'] = scores

            return report
        except Exception as e:
            self.logger.error(f"Error generating report: {str(e)}")
            return None

    def _calculate_compliance(self):
        """Calculate compliance scores"""
        scores = {
            'total_checks': 0,
            'passed_checks': 0,
            'failed_checks': 0,
            'not_applicable': 0,
            'categories': {}
        }

        try:
            def process_results(results, category=''):
                for key, value in results.items():
                    if isinstance(value, dict):
                        cat = f"{category}/{key}" if category else key
                        scores['categories'][cat] = {
                            'total': 0,
                            'passed': 0,
                            'failed': 0,
                            'na': 0
                        }
                        process_results(value, cat)
                    else:
                        scores['total_checks'] += 1
                        if category:
                            scores['categories'][category]['total'] += 1
                        
                        if value is None:
                            scores['not_applicable'] += 1
                            if category:
                                scores['categories'][category]['na'] += 1
                        elif value is True:
                            scores['passed_checks'] += 1
                            if category:
                                scores['categories'][category]['passed'] += 1
                        else:
                            scores['failed_checks'] += 1
                            if category:
                                scores['categories'][category]['failed'] += 1

            process_results(self.results)

            # Calculate percentages
            total_applicable = (
                scores['total_checks'] - scores['not_applicable']
            )
            if total_applicable > 0:
                scores['compliance_percentage'] = (
                    scores['passed_checks'] / total_applicable * 100
                )
                
                for category in scores['categories'].values():
                    total_applicable = (
                        category['total'] - category['na']
                    )
                    if total_applicable > 0:
                        category['compliance_percentage'] = (
                            category['passed'] / total_applicable * 100
                        )
                    else:
                        category['compliance_percentage'] = 0
            else:
                scores['compliance_percentage'] = 0

        except Exception as e:
            self.logger.error(f"Error calculating compliance: {str(e)}")

        return scores

    def save_report(self, report, output_file):
        """Save compliance report to file"""
        try:
            with open(output_file, 'w') as f:
                json.dump(report, f, indent=4)
            self.logger.info(f"Report saved to {output_file}")
        except Exception as e:
            self.logger.error(f"Error saving report: {str(e)}")

    def print_summary(self, report):
        """Print compliance summary"""
        self.logger.info("\nCompliance Check Summary:")
        self.logger.info(f"Target: {report['target']}")
        self.logger.info(f"Standard: {report['standard']}")
        self.logger.info(
            f"Operating System: {report['system']['os']} "
            f"{report['system']['version']}"
        )
        
        if 'compliance' in report:
            compliance = report['compliance']
            self.logger.info("\nCompliance Summary:")
            self.logger.info(
                f"Overall Compliance: "
                f"{compliance['compliance_percentage']:.1f}%"
            )
            self.logger.info(f"Total Checks: {compliance['total_checks']}")
            self.logger.info(
                f"Passed: {compliance['passed_checks']}"
            )
            self.logger.info(
                f"Failed: {compliance['failed_checks']}"
            )
            self.logger.info(
                f"Not Applicable: {compliance['not_applicable']}"
            )
            
            self.logger.info("\nCategory Compliance:")
            for category, scores in compliance['categories'].items():
                if scores['total'] > 0:
                    self.logger.info(
                        f"\n{category}:"
                    )
                    self.logger.info(
                        f"Compliance: {scores['compliance_percentage']:.1f}%"
                    )
                    self.logger.info(
                        f"Passed: {scores['passed']}/{scores['total']}"
                    )

    def start_check(self, output_file=None):
        """Start compliance check"""
        self.logger.info("Starting compliance check")
        self.logger.info(f"Target: {self.target}")
        self.logger.info(f"Standard: {self.standard}")
        
        try:
            # Generate and save report
            report = self.generate_report()
            if report and output_file:
                self.save_report(report, output_file)
            if report:
                self.print_summary(report)
                
        except KeyboardInterrupt:
            self.logger.info("Check interrupted by user")
        except Exception as e:
            self.logger.error(f"Error during check: {str(e)}")

if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Compliance Checker"
    )
    parser.add_argument(
        "target",
        help="Target system or configuration"
    )
    parser.add_argument(
        "--standard",
        choices=['pci-dss', 'hipaa', 'iso27001'],
        default='pci-dss',
        help="Compliance standard to check against"
    )
    parser.add_argument(
        "--output",
        help="Output file for compliance report"
    )
    args = parser.parse_args()

    checker = ComplianceChecker(args.target, args.standard)
    checker.start_check(args.output)
